//Example from: https://github.com/VolosR/PCBDesignClock

#include <TFT_eSPI.h>
#include <WiFi.h>
#include "time.h"
#include "fonts.h"
#include "fonts2.h"
#include "multiple.h"
#include "pin_config.h"
#include <stdio.h>
#include <string.h>

// Size of sprite image for the scrolling text, this requires ~14 Kbytes of RAM
#define IWIDTH  320
#define IHEIGHT 170

TFT_eSPI    tft = TFT_eSPI();         // Create object "tft"
TFT_eSprite img = TFT_eSprite(&tft);  // Create Sprite object "img" with pointer to "tft" object
//                                    // the pointer is used by pushSprite() to push it onto the TFT

// -------------------------------------------------------------------------
// Setup
// -------------------------------------------------------------------------

const char *ssid     = "XXXXXXXXXXX";
const char *password = "XXXXXXXXXXX";

const char *ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = -21600;         //time zone * 3600 , my time zone is  -5 GTM
const int   daylightOffset_sec = 3600;

char timeHour[3];
char timeMin[3];
char timeSec[3];
char day[3];
char month[10];
char year[5];
char timeWeekDay[10];
int dayInWeek;

int brightnesses[7] = {1, 15, 30, 60, 120, 200, 250};
int bright = 3;
int deb = 0;
int deb2 = 0;

unsigned long currTime = 0;
int period = 9000;

short LF = 10;        // ASCII linefeed

bool CLOCKMODE = true;
String MESSAGETEXT;
char MESSAGE2[1024];
char ScoreA[1024];
char ScoreB[1024];
int stringlength;
int stringlength2;
char *token;
char *team1, *team2, *score1, *score2;
int value;
int value1;
int value2;

void setup() 
{
  pinMode(15, OUTPUT);
  digitalWrite(15, HIGH);

  pinMode(0, INPUT_PULLUP);
  pinMode(14, INPUT_PULLUP);
  tft.init();
  tft.invertDisplay(1);
  tft.setRotation(3);

  tft.fillScreen(TFT_BLACK);
  Serial.begin(115200);

  ledcSetup(0, 10000, 8);
  ledcAttachPin(38, 0);
  ledcWrite(0, brightnesses[bright]);

  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  } configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
}

// -------------------------------------------------------------------------
// Main loop
// -------------------------------------------------------------------------
void loop() {
    
  while (Serial.available()) {
    String message = Serial.readStringUntil(LF); // read serial data
    if (message == "MODE") {
      CLOCKMODE=!CLOCKMODE;
      Serial.print ("q");
      while (Serial.available() == 0) {
        // Do nothing, just wait
      }
      String message = Serial.readStringUntil(LF); // read serial data
      MESSAGETEXT=message;
      Serial.print ("q");
    }
  }
    
  while (CLOCKMODE == false)
  {
    img.createSprite(IWIDTH, IHEIGHT);
    img.setSwapBytes(true);

    strcpy(MESSAGE2, MESSAGETEXT.c_str());
    // Get the first token
    token = strtok(MESSAGE2, " ");
    // Assign the first four tokens to variables
    team1 = token;
    token = strtok(NULL, " ");
    score1 = token;
    token = strtok(NULL, " ");
    team2 = token;    
    token = strtok(NULL, " ");
    score2 = token;

    stringlength = strlen(score1);
    stringlength2 = strlen(score2);
    value1=GetTeamInt(team1);
    value2=GetTeamInt(team2);
    if (value1 != 30) {
        ledcWrite(0, brightnesses[6]);
    }
    for (int pos = IWIDTH; pos > ((stringlength+stringlength2)*-100)-460; (pos=pos-4))
    {
      build_banner(value1, value2, score1, score2, stringlength, pos);
      img.pushSprite(0, 0);
    }
    while (Serial.available()) {
      String message = Serial.readStringUntil(LF); // read serial data
      if (message == "MODE") {
        CLOCKMODE=!CLOCKMODE;
        ledcWrite(0, brightnesses[bright]);
      }else{
        MESSAGETEXT = message;
        Serial.print ("q");
      }
    }
    img.deleteSprite();
  }
  if (digitalRead(0) == 0) { //if left button pressed , change brightness
    if (deb == 0) {
        deb = 1; bright++; if (bright == 7) bright = 0;
        ledcWrite(0, brightnesses[bright]);
    }
  } else deb = 0;
  if (digitalRead(14) == 0) { //if left button pressed , do nothing
    if (deb2 == 0) {
      deb2 = 1;
    }
    } else deb2 = 0;
  if (millis() > currTime + period) { //update time, and chose next frame every "period" of time
    currTime = millis(); getTime();
  }
  img.createSprite(IWIDTH, IHEIGHT);
  draw();
  img.pushSprite(0, 0);
  img.deleteSprite();   
}

// #########################################################################
// Build the scrolling sprite image from scratch, draw text at x = xpos
// #########################################################################

void build_banner(int Team1, int Team2, String Score1, String Score2, float scorelength, int xpos)
{
  img.setColorDepth(16);
  img.fillSprite(TFT_BLACK);
  img.pushImage(xpos, 15, 140, 140, frames[Team1]);
  if (Team1 == 30) {
    img.setFreeFont(&Arimo_Regular_80);
    img.setTextColor(TFT_LIGHTGREY);
    img.setTextWrap(false);       // Turn of wrap so we can print past end of sprite
    img.setTextSize(2);
    img.setCursor(xpos+180, 130);  // Print text at xpos
    img.print(Score1);
    if (scorelength > 6) {
      scorelength = scorelength -1.3;
    } else {
      scorelength = scorelength -.6;
    }
    img.setCursor(xpos+200+(scorelength*100), 130);  // Print text at xpos
    img.print("/");
    img.setCursor(xpos+280+(scorelength*100), 130);  // Print text at xpos
    img.print(Score2);
  } else {
    img.pushImage(xpos+(scorelength*100)+280, 15, 140, 140, frames[Team2]);
    img.setFreeFont(&Arimo_Regular_80);
    img.setTextColor(TFT_YELLOW);
    img.setTextWrap(false);       // Turn of wrap so we can print past end of sprite
    img.setTextSize(2);
    img.setCursor(xpos+180, 130);  // Print text at xpos
    img.print(Score1);
    img.setCursor(xpos+200+(scorelength*100), 130);  // Print text at xpos
    img.setTextColor(TFT_RED);
    img.print("/");
    img.setTextColor(TFT_YELLOW);
    img.setCursor(xpos+460+(scorelength*100), 130);  // Print text at xpos
    img.print(Score2);
  }
}

void draw()
{
    img.setColorDepth(16);
    img.fillSprite(TFT_BLACK);
    img.setFreeFont(&DSEG7_Classic_Bold_84);
    img.setTextColor(TFT_RED);
    img.setTextWrap(true);
    img.setTextSize(1);
    img.drawString(String(timeHour) + ":" + String(timeMin), 12, 42);
}

void getTime()    // get time from server
{
    struct tm timeinfo;
    if (!getLocalTime(&timeinfo)) {
        return;
    }
    strftime(timeHour, 3, "%I", &timeinfo);
    strftime(timeMin, 3, "%M", &timeinfo);
    strftime(timeSec, 3, "%S", &timeinfo);
    strftime(timeWeekDay, 10, "%A", &timeinfo);
    strftime(day, 3, "%d", &timeinfo);
    strftime(month, 10, "%B", &timeinfo);
    strftime(year, 5, "%Y", &timeinfo);
}

int GetTeamInt(char *array) {
	if (strcmp(array, "76ers") == 0) {
		value=0;
	} else if (strcmp(array, "Bucks") == 0) {
		value=1;
	} else if (strcmp(array, "Bulls") == 0) {
		value=2;
	} else if (strcmp(array, "Cavaliers") == 0) {
		value=3;
	} else if (strcmp(array, "Celtics") == 0) {
		value=4;
	} else if (strcmp(array, "Clippers") == 0) {
		value=5;
	} else if (strcmp(array, "Grizzlies") == 0) {
		value=6;
	} else if (strcmp(array, "Hawks") == 0) {
		value=7;
	} else if (strcmp(array, "Heat") == 0) {
		value=8;
	} else if (strcmp(array, "Hornets") == 0) {
		value=9;
	} else if (strcmp(array, "Jazz") == 0) {
		value=10;
	} else if (strcmp(array, "Kings") == 0) {
		value=11;
	} else if (strcmp(array, "Knicks") == 0) {
		value=12;
	} else if (strcmp(array, "Lakers") == 0) {
		value=13;
	} else if (strcmp(array, "Magic") == 0) {
		value=14;
	} else if (strcmp(array, "Mavericks") == 0) {
		value=15;
	} else if (strcmp(array, "Nets") == 0) {
		value=16;
	} else if (strcmp(array, "Nuggets") == 0) {
		value=17;
	} else if (strcmp(array, "Pacers") == 0) {
		value=18;
	} else if (strcmp(array, "Pelicans") == 0) {
		value=19;
	} else if (strcmp(array, "Pistons") == 0) {
		value=20;
	} else if (strcmp(array, "Raptors") == 0) {
		value=21;
	} else if (strcmp(array, "Rockets") == 0) {
		value=22;
	} else if (strcmp(array, "Spurs") == 0) {
		value=23;
	} else if (strcmp(array, "Suns") == 0) {
		value=24;
	} else if (strcmp(array, "Thunder") == 0) {
		value=25;
	} else if (strcmp(array, "Timberwolves") == 0) {
		value=26;
	} else if (strcmp(array, "Trail_Blazers") == 0) {
		value=27;
	} else if (strcmp(array, "Warriors") == 0) {
		value=28;
	} else if (strcmp(array, "Wizards") == 0) {
		value=29;
  } else if (strcmp(array, "Kodi") == 0) {
		value=30;
  }
	return value;
}